@@ -1,14 +1,13 @@
 /*
  * This file is part of osso-application-installer
  *
- * Copyright (C) 2005 Nokia Corporation.
+ * Copyright (C) 2005, 2006 Nokia Corporation.  All Right reserved.
  *
  * Contact: Marius Vollmer <marius.vollmer@nokia.com>
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of the GNU General Public License version
+ * 2 as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,7 +29,8 @@
 void create_menu (GtkMenu *main);
 void set_details_menu_sensitive (bool);
 void set_search_menu_sensitive (bool);
-void set_operation_menu_label (const gchar *label, bool sensitive);
+void set_operation_menu_label (const gchar *label, bool sensitive,
+			       const gchar *insens);
 
 void set_fullscreen_menu_check (bool f);
 
