@@ -69,7 +69,6 @@ struct repo_closure {
   GtkTreeView *tree;
   GtkListStore *store;
 
-  GtkWindow *dialog;
   GtkWidget *edit_button;
   GtkWidget *delete_button;
 
@@ -300,6 +299,7 @@ repo_edit_response (GtkDialog *dialog, gint response, gpointer clos)
 
   delete c;
 
+  pop_dialog_parent ();
   gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
@@ -345,8 +345,7 @@ add_entry (GtkWidget *box, GtkSizeGroup *group,
 }
 
 static void
-show_repo_edit_dialog (GtkWindow *parent,
-		       repo_line *r, bool isnew, bool readonly)
+show_repo_edit_dialog (repo_line *r, bool isnew, bool readonly)
 {
   GtkWidget *dialog, *vbox, *caption;
   GtkSizeGroup *group;
@@ -370,7 +369,7 @@ show_repo_edit_dialog (GtkWindow *parent,
     {
       GtkWidget *button;
 
-      dialog = gtk_dialog_new_with_buttons (title, parent,
+      dialog = gtk_dialog_new_with_buttons (title, get_dialog_parent (),
 					    GTK_DIALOG_MODAL,
 					    NULL);
 
@@ -382,7 +381,7 @@ show_repo_edit_dialog (GtkWindow *parent,
       gtk_widget_grab_focus (button);
     }
   else
-    dialog = gtk_dialog_new_with_buttons (title, parent,
+    dialog = gtk_dialog_new_with_buttons (title, get_dialog_parent (),
 					  GTK_DIALOG_MODAL,
 					  _("ai_bd_new_repository_ok"),
 					  GTK_RESPONSE_OK,
@@ -390,6 +389,8 @@ show_repo_edit_dialog (GtkWindow *parent,
 					  GTK_RESPONSE_CANCEL,
 					  NULL);
 
+  push_dialog_parent (dialog);
+
   gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 
   // XXX - there is no help for the "edit" version of this dialog.
@@ -477,7 +478,7 @@ add_new_repo (repo_closure *c)
   *c->lastp = r;
   c->lastp = &r->next;
 
-  show_repo_edit_dialog (c->dialog, r, true, false);
+  show_repo_edit_dialog (r, true, false);
 }
 
 static void
@@ -576,7 +577,7 @@ repo_response (GtkDialog *dialog, gint response, gpointer clos)
 
       if (r->essential)
 	irritate_user (_("ai_ib_unable_edit"));
-      show_repo_edit_dialog (c->dialog, r, false, r->essential);
+      show_repo_edit_dialog (r, false, r->essential);
 
       return;
     }
@@ -606,6 +607,7 @@ repo_response (GtkDialog *dialog, gint response, gpointer clos)
 	}
       
       delete c;
+      pop_dialog_parent ();
       gtk_widget_destroy (GTK_WIDGET (dialog));
     }
 }
@@ -663,7 +665,6 @@ repo_row_activated (GtkTreeView *treeview,
 		    GtkTreeViewColumn *column,
 		    gpointer data)
 {
-  repo_closure *c = (repo_closure *)data;
   GtkTreeModel *model = gtk_tree_view_get_model (treeview);
   GtkTreeIter iter;
 
@@ -676,7 +677,7 @@ repo_row_activated (GtkTreeView *treeview,
 
       if (r->essential)
 	irritate_user (_("ai_ib_unable_edit"));
-      show_repo_edit_dialog (c->dialog, r, false, r->essential);
+      show_repo_edit_dialog (r, false, r->essential);
     }
 }
 
@@ -810,8 +811,7 @@ maybe_add_new_repo_details (void *data)
 {
   repo_add_closure *ac = (repo_add_closure *)data;
 
-  show_repo_edit_dialog (get_main_window (),
-			 ac->new_repo, true, true);
+  show_repo_edit_dialog (ac->new_repo, true, true);
 }
 
 static void
@@ -907,12 +907,11 @@ sources_list_reply (int cmd, apt_proto_decoder *dec, void *data)
     {
       GtkWidget *dialog = gtk_dialog_new ();
 
-      c->dialog = GTK_WINDOW (dialog);
-
       gtk_window_set_title (GTK_WINDOW (dialog), _("ai_ti_repository"));
-      gtk_window_set_transient_for (GTK_WINDOW (dialog), get_main_window ());
+      gtk_window_set_transient_for (GTK_WINDOW (dialog), get_dialog_parent ());
       gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-      
+      push_dialog_parent (dialog);
+
       gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
       gtk_dialog_add_button (GTK_DIALOG (dialog), 
 			     _("ai_bd_repository_new"), REPO_RESPONSE_NEW);
@@ -979,6 +978,7 @@ maybe_add_repo (const char *name, const char *deb_line,	bool for_install,
 static void
 pill_response (GtkDialog *dialog, gint response, gpointer unused)
 {
+  pop_dialog_parent ();
   gtk_widget_destroy (GTK_WIDGET (dialog));
 
   if (red_pill_mode != (response == GTK_RESPONSE_YES))
@@ -995,11 +995,12 @@ ask_the_pill_question ()
   GtkWidget *dialog;
 
   dialog =
-    hildon_note_new_confirmation_add_buttons (get_main_window (), 
+    hildon_note_new_confirmation_add_buttons (get_dialog_parent (), 
 					      "Which pill?",
 					      "Red", GTK_RESPONSE_YES,
 					      "Blue", GTK_RESPONSE_NO,
 					      NULL);
+  push_dialog_parent (dialog);
   g_signal_connect (dialog, "response",
 		    G_CALLBACK (pill_response), NULL);
   gtk_widget_show_all (dialog);
