@@ -1,14 +1,13 @@
 /*
  * This file is part of osso-application-installer
  *
- * Copyright (C) 2005 Nokia Corporation.
+ * Copyright (C) 2005, 2006 Nokia Corporation.  All Right reserved.
  *
  * Contact: Marius Vollmer <marius.vollmer@nokia.com>
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of the GNU General Public License version
+ * 2 as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,6 +22,8 @@
  */
 
 #define DBUS_API_SUBJECT_TO_CHANGE
+#include <dbus/dbus.h>
+#include <dbus/dbus-glib-lowlevel.h>
 
 #include <string.h>
 #include <unistd.h>
@@ -41,7 +42,7 @@
 #include <hildon-widgets/hildon-banner.h>
 #include <gdk/gdkkeysyms.h>
 #include <hildon-widgets/hildon-defines.h>
-#include <osso-ic.h>
+#include <conic.h>
 #include <libgnomevfs/gnome-vfs.h>
 
 extern "C" {
@@ -57,9 +58,51 @@ extern "C" {
 
 #define _(x) gettext (x)
 
+/* For getting and tracking the Bluetooth name
+ */
+#define BTNAME_SERVICE                  "org.bluez"
+#define BTNAME_REQUEST_IF               "org.bluez.Adapter"
+#define BTNAME_SIGNAL_IF                "org.bluez.Adapter"
+#define BTNAME_REQUEST_PATH             "/org/bluez/hci0"
+#define BTNAME_SIGNAL_PATH              "/org/bluez/hci0"
+
+#define BTNAME_REQ_GET                  "GetName"
+#define BTNAME_SIG_CHANGED              "NameChanged"
+
+#define BTNAME_MATCH_RULE "type='signal',interface='" BTNAME_SIGNAL_IF \
+                          "',member='" BTNAME_SIG_CHANGED "'"
+
+static GSList *dialog_parents = NULL;
+
+GtkWindow *
+get_dialog_parent ()
+{
+  if (dialog_parents)
+    return GTK_WINDOW (dialog_parents->data);
+  else
+    return NULL;
+}
+
+void
+push_dialog_parent (GtkWidget *w)
+{
+  dialog_parents = g_slist_prepend (dialog_parents, w);
+}
+
+void
+pop_dialog_parent ()
+{
+  if (dialog_parents)
+    {
+      GSList *old = dialog_parents;
+      dialog_parents = dialog_parents->next;
+      g_slist_free_1 (old);
+    }
+}
+
 struct ayn_closure {
   package_info *pi;
-  bool installed;
+  detail_kind kind;
   void (*cont) (bool res, void *data);
   void (*details) (void *data);
   void *data;
@@ -73,7 +116,7 @@ yes_no_response (GtkDialog *dialog, gint response, gpointer clos)
   if (response == 1)
     {
       if (c->pi)
-	show_package_details (c->pi, c->installed, false);
+	show_package_details (c->pi, c->kind, false);
       else if (c->details)
 	c->details (c->data);
       return;
@@ -85,6 +128,7 @@ yes_no_response (GtkDialog *dialog, gint response, gpointer clos)
     c->pi->unref ();
   delete c;
 
+  pop_dialog_parent ();
   gtk_widget_destroy (GTK_WIDGET (dialog));
   cont (response == GTK_RESPONSE_OK, data);
 }
@@ -101,7 +145,8 @@ ask_yes_no (const gchar *question,
   c->details = NULL;
   c->data = data;
 
-  dialog = hildon_note_new_confirmation (get_main_window (), question);
+  dialog = hildon_note_new_confirmation (get_dialog_parent (), question);
+  push_dialog_parent (dialog);
 
   g_signal_connect (dialog, "response",
 		    G_CALLBACK (yes_no_response), c);
@@ -122,11 +167,12 @@ ask_custom (const gchar *question,
   c->data = data;
 
   dialog = hildon_note_new_confirmation_add_buttons 
-    (get_main_window (),
+    (get_dialog_parent (),
      question,
      ok_label, GTK_RESPONSE_OK,
      cancel_label, GTK_RESPONSE_CANCEL,
      NULL);
+  push_dialog_parent (dialog);
 
   g_signal_connect (dialog, "response",
 		    G_CALLBACK (yes_no_response), c);
@@ -136,7 +182,7 @@ ask_custom (const gchar *question,
 void
 ask_yes_no_with_details (const gchar *title,
 			 const gchar *question,
-			 package_info *pi, bool installed,
+			 package_info *pi, detail_kind kind,
 			 void (*cont) (bool res, void *data),
 			 void *data)
 {
@@ -144,20 +190,22 @@ ask_yes_no_with_details (const gchar *title,
   ayn_closure *c = new ayn_closure;
   c->pi = pi;
   pi->ref ();
-  c->installed = installed;
+  c->kind = kind;
   c->cont = cont;
   c->details = NULL;
   c->data = data;
 
   dialog = gtk_dialog_new_with_buttons
     (title,
-     get_main_window (),
+     get_dialog_parent (),
      GTK_DIALOG_MODAL,
      _("ai_bd_confirm_ok"),      GTK_RESPONSE_OK,
      _("ai_bd_confirm_details"), 1,
      _("ai_bd_confirm_cancel"),  GTK_RESPONSE_CANCEL,
      NULL);
+  push_dialog_parent (dialog);
 
+  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox),
 		     gtk_label_new (question));
 
@@ -182,15 +230,19 @@ ask_yes_no_with_arbitrary_details (const gchar *title,
 
   dialog = gtk_dialog_new_with_buttons
     (title,
-     get_main_window (),
+     get_dialog_parent (),
      GTK_DIALOG_MODAL,
      _("ai_bd_confirm_ok"),      GTK_RESPONSE_OK,
      _("ai_bd_confirm_details"), 1,
      _("ai_bd_confirm_cancel"),  GTK_RESPONSE_CANCEL,
      NULL);
+  push_dialog_parent (dialog);
 
+  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+  GtkWidget *label = gtk_label_new (question);
+  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox),
-		     gtk_label_new (question));
+		     label);
 
   g_signal_connect (dialog, "response",
 		    G_CALLBACK (yes_no_response), c);
@@ -202,6 +254,7 @@ static bool currently_annoying_user = false;
 static void
 annoy_user_response (GtkDialog *dialog, gint response, gpointer data)
 {
+  pop_dialog_parent ();
   gtk_widget_destroy (GTK_WIDGET (dialog));
   currently_annoying_user = false;
 }
@@ -214,7 +267,8 @@ annoy_user (const gchar *text)
 
   GtkWidget *dialog;
 
-  dialog = hildon_note_new_information (get_main_window (), text);
+  dialog = hildon_note_new_information (get_dialog_parent (), text);
+  push_dialog_parent (dialog);
   g_signal_connect (dialog, "response", 
 		    G_CALLBACK (annoy_user_response), NULL);
   gtk_widget_show_all (dialog);
@@ -223,7 +277,7 @@ annoy_user (const gchar *text)
 
 struct auwd_closure {
   package_info *pi;
-  bool installed;
+  detail_kind kind;
 };
 
 static void
@@ -234,10 +288,11 @@ annoy_user_with_details_response (GtkDialog *dialog, gint response,
 
   if (response == 1)
     {
-      show_package_details (c->pi, c->installed, true);
+      show_package_details (c->pi, c->kind, true);
     }
   else
     {
+      pop_dialog_parent ();
       gtk_widget_destroy (GTK_WIDGET (dialog));
       currently_annoying_user = false;
       c->pi->unref ();
@@ -247,17 +302,16 @@ annoy_user_with_details_response (GtkDialog *dialog, gint response,
 
 void
 annoy_user_with_details (const gchar *text,
-			 package_info *pi, bool installed)
+			 package_info *pi, detail_kind kind)
 {
   if (currently_annoying_user)
     return;
 
-  GtkWidget *dialog, *action_area;
-  GtkWidget *details_button;
+  GtkWidget *dialog;
   auwd_closure *c = new auwd_closure;
-  gint response;
 
-  dialog = hildon_note_new_information (get_main_window (), text);
+  dialog = hildon_note_new_information (get_dialog_parent (), text);
+  push_dialog_parent (dialog);
 
   {
     // XXX - the buttons should be "Details" "Close", so we remove the
@@ -271,12 +325,13 @@ annoy_user_with_details (const gchar *text,
     g_list_free (kids);
 
     gtk_dialog_add_button (GTK_DIALOG (dialog), _("ai_ni_bd_details"), 1);
-    gtk_dialog_add_button (GTK_DIALOG (dialog), _("ai_ni_bd_close"), 2);
+    gtk_dialog_add_button (GTK_DIALOG (dialog), _("ai_ni_bd_close"),
+			   GTK_RESPONSE_CANCEL);
   }
 
   pi->ref ();
   c->pi = pi;
-  c->installed = installed;
+  c->kind = kind;
   g_signal_connect (dialog, "response", 
 		    G_CALLBACK (annoy_user_with_details_response), c);
   gtk_widget_show_all (dialog);
@@ -292,6 +347,7 @@ static void
 annoy_user_with_log_response (GtkDialog *dialog, gint response,
 			      gpointer data)
 {
+  pop_dialog_parent ();
   gtk_widget_destroy (GTK_WIDGET (dialog));
   currently_annoying_user = false;
 
@@ -305,10 +361,10 @@ annoy_user_with_log (const gchar *text)
   if (currently_annoying_user)
     return;
 
-  GtkWidget *dialog, *action_area;
-  GtkWidget *details_button;
+  GtkWidget *dialog;
 
-  dialog = hildon_note_new_information (get_main_window (), text);
+  dialog = hildon_note_new_information (get_dialog_parent (), text);
+  push_dialog_parent (dialog);
 #if 0
   gtk_dialog_add_button (GTK_DIALOG (dialog), "Log", 1);
 #endif
@@ -382,12 +438,14 @@ scare_user_with_legalese (bool sure,
 
   dialog = gtk_dialog_new_with_buttons
     (_("ai_ti_notice"),
-     get_main_window (),
+     get_dialog_parent (),
      GTK_DIALOG_MODAL,
      _("ai_bd_notice_ok"),      GTK_RESPONSE_OK,
      _("ai_bd_notice_cancel"),  GTK_RESPONSE_CANCEL,
      NULL);
+  push_dialog_parent (dialog);
 
+  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
   const char *text = (sure
 		      ? _("ai_nc_non_verified_package")
 		      : _("ai_nc_unsure_package"));
@@ -467,17 +525,45 @@ progress_was_cancelled ()
   return progress_cancelled;
 }
 
+void
+reset_progress_was_cancelled ()
+{
+  progress_cancelled = false;
+  cancel_count = 0;
+}
+
 static void
-create_progress (bool with_cancel)
+create_progress (const gchar *title, bool with_cancel)
 {
   if (progress_dialog)
-    gtk_widget_destroy (progress_dialog);
+    {
+      pop_dialog_parent ();
+      gtk_widget_destroy (progress_dialog);
+      progress_dialog = NULL;
+      progress_bar = NULL;
+    }
 
+
+  // XXX - we make the title slightly longer so that there is a bit
+  //       room to grow without having to resize the dialog or risking
+  //       ellipsization.  This only matters when the total download
+  //       size changes during a download and it that case it would be
+  //       better to resize the dialog, but that doesn't seem to
+  //       happen...
+  //
+  //       The "XXXX" is never shown since we set a real title
+  //       immediately.
+  
+  gchar *longer_title = g_strconcat (title, "XXX", NULL);
   progress_bar = GTK_PROGRESS_BAR (gtk_progress_bar_new ());
   progress_dialog =
-    hildon_note_new_cancel_with_progress_bar (get_main_window (),
-					      _("ai_nw_updating_list"),
+    hildon_note_new_cancel_with_progress_bar (get_dialog_parent (),
+					      longer_title,
 					      progress_bar);
+  push_dialog_parent (progress_dialog);
+  g_free (longer_title);
+  g_object_set (progress_dialog, "description", title, NULL);
+
   if (!with_cancel)
     {
       GtkWidget *box = GTK_DIALOG (progress_dialog)->action_area;
@@ -487,13 +573,13 @@ create_progress (bool with_cancel)
       g_list_free (kids);
     }
   else
-    g_signal_connect (progress_dialog, "response",
-		      G_CALLBACK (cancel_response), NULL);
+    {
+      respond_on_escape (GTK_DIALOG (progress_dialog), GTK_RESPONSE_CANCEL);
+      g_signal_connect (progress_dialog, "response",
+			G_CALLBACK (cancel_response), NULL);
+    }
 
   gtk_widget_show (progress_dialog);
-
-  progress_cancelled = false;
-  cancel_count = 0;
 }
 
 void
@@ -505,52 +591,85 @@ set_general_progress_title (const char *title)
 void
 show_progress (const char *title)
 {
-  create_progress (FALSE);
+  create_progress (title, FALSE);
 
   set_general_progress_title (title);
   current_status_operation = op_general;
   set_progress (op_general, -1, 0);
+
+  reset_progress_was_cancelled ();
 }
 
 void
 set_progress (apt_proto_operation op, int already, int total)
 {
-  const char *title;
+  const char *title = NULL;
 
-  if (op != current_status_operation
-      && (progress_dialog || op != op_updating_cache))
-    {
-      hide_progress ();
-      create_progress (op == op_downloading);
-    }
+  // Determine if we need a new title
 
   if (op == op_downloading)
     {
+      // The downloading title can change dynamically when the total
+      // changes.
+
       static int last_total;
       static char *dynlabel = NULL;
 
-      if (dynlabel == NULL || total != last_total)
+      if (dynlabel == NULL
+	  || total != last_total
+	  || current_status_operation != op_downloading)
 	{
 	  char size_buf[20];
 	  size_string_detailed (size_buf, 20, total);
 	  g_free (dynlabel);
-	  dynlabel =  g_strdup_printf (_("ai_nw_downloading"), size_buf);
+	  dynlabel = g_strdup_printf (_("ai_nw_downloading"), size_buf);
+	  title = dynlabel;
 	}
-      title = dynlabel;
     }
-  else if (op == op_updating_cache)
-    title = _("ai_nw_updating_list");
-  else
-    title = general_title;
-
-  current_status_operation = op;
+  else if (op != current_status_operation)
+    {
+      // Otherwise the title only changes when the operation changes
+      
+      if (op == op_updating_cache)
+	title = _("ai_nw_updating_list");
+      else
+	title = general_title;
+    }
 
   // printf ("STATUS: %s -- %f\n", title, fraction);
 
-  if (progress_dialog)
+  // We might need to change the progress title or we might need to
+  // create the whole dialog here.
+  //
+  // XXX - The dialog will only be created when the new operation is
+  // not op_updating_cache since apt-worker outputs progress messages
+  // with that operation that we don't want to show (for example,
+  // during startup).  The whole progress logic needs to be better
+  // defined and apt-worker made more silent, etc.
+
+  if (progress_dialog || op != op_updating_cache)
     {
-      g_object_set (progress_dialog, "description", title, NULL);
-      
+      // 'title' is non-NULL when it needs to change.  Since the
+      // dialog does not resize when its contents changes, we simply
+      // recreate it, except when the operation is op_downloading.  In
+      // that case, title changes are frequent and recreating the
+      // dialog every time is annoying.  In that case, we simply set
+      // the new description.  CREATE_PROGRESS takes care that there
+      // is a bit of space for the title to grow.
+
+      if (title
+	  && op == op_downloading
+	  && current_status_operation == op_downloading
+	  && progress_dialog)
+	{
+	  g_object_set (progress_dialog, "description", title, NULL);
+	}
+      else if (title || progress_dialog == NULL)
+	{
+	  hide_progress ();
+	  create_progress (title, op == op_downloading);
+	}
+
       if (already >= 0)
 	{
 	  stop_pulsing ();
@@ -560,6 +679,8 @@ set_progress (apt_proto_operation op, int already, int total)
       else
 	start_pulsing ();
     }
+
+  current_status_operation = op;
 }
 
 void
@@ -568,42 +689,35 @@ hide_progress ()
   if (progress_dialog)
     {
       stop_pulsing ();
+      pop_dialog_parent ();
+      gtk_widget_destroy (GTK_WIDGET(progress_bar));
       gtk_widget_destroy (progress_dialog);
       progress_dialog = NULL;
+      progress_bar = NULL;
     }
   current_status_operation = op_general;
 }
 
 static GtkWidget *updating_banner = NULL;
 static int updating_level = 0;
+static const char *updating_label = NULL;
+static bool allow_updating_banner = true;
 
-static gboolean
-updating_timeout (gpointer unused)
+static void
+refresh_updating_banner ()
 {
-  if (updating_banner == NULL && updating_level > 0)
+  bool show_it = (updating_level > 0 && allow_updating_banner);
+
+  if (show_it && updating_banner == NULL)
     {
       updating_banner = 
 	hildon_banner_show_animation (GTK_WIDGET (get_main_window ()),
 				      NULL,
-				      dgettext ("hildon-common-strings",
-						"ckdg_pb_updating"));
+				      updating_label);
       g_object_ref (updating_banner);
     }
-  return FALSE;
-}
 
-void
-show_updating ()
-{
-  gtk_timeout_add (2000, updating_timeout, NULL);
-  updating_level++;
-}
-
-void
-hide_updating ()
-{
-  updating_level--;
-  if (updating_banner && updating_level == 0)
+  if (!show_it && updating_banner != NULL)
     {
       gtk_widget_destroy (updating_banner);
       g_object_unref (updating_banner);
@@ -611,16 +725,74 @@ hide_updating ()
     }
 }
 
+static gboolean
+updating_timeout (gpointer unused)
+{
+  updating_level++;
+  refresh_updating_banner ();
+  return FALSE;
+}
+
+void
+show_updating (const char *label)
+{
+  if (label == NULL)
+    label = dgettext ("hildon-common-strings",
+		      "ckdg_pb_updating");
+
+  updating_label = label;
+
+  // We must never cancel this timeout since otherwise the
+  // UPDATING_LEVEL will get out of sync.
+  gtk_timeout_add (2000, updating_timeout, NULL);
+}
+
+void
+hide_updating ()
+{
+  updating_level--;
+  refresh_updating_banner ();
+}
+
+void
+allow_updating ()
+{
+  allow_updating_banner = true;
+  refresh_updating_banner ();
+}
+
+void
+prevent_updating ()
+{
+  allow_updating_banner = false;
+  refresh_updating_banner ();
+}
+
 static PangoFontDescription *
-get_small_font ()
+get_small_font (GtkWidget *widget)
 {
   static PangoFontDescription *small_font = NULL;
+  gint size = 0;
 
   if (small_font == NULL)
     {
-      // XXX - do it nicer...
-      small_font = pango_font_description_from_string ("Nokia Sans 13");
+      GtkStyle *fontstyle = NULL;
+
+      fontstyle = gtk_rc_get_style_by_paths (gtk_widget_get_settings (GTK_WIDGET(widget)),
+                                             "osso-SystemFont", NULL,
+                                             G_TYPE_NONE);
+  
+      if (fontstyle) {
+        small_font = pango_font_description_copy (fontstyle->font_desc);
+      } else {
+        small_font = pango_font_description_from_string ("Nokia Sans 16.75");
+      }
+      size = pango_font_description_get_size(small_font);
+      size = gint (size * PANGO_SCALE_SMALL);
+      pango_font_description_set_size(small_font, size);
+       
     }
+
   return small_font;
 }
 
@@ -650,7 +822,7 @@ make_small_text_view (const char *text)
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
 				  GTK_POLICY_AUTOMATIC,
 				  GTK_POLICY_AUTOMATIC);
-  gtk_widget_modify_font (view, get_small_font ());
+  gtk_widget_modify_font (view, get_small_font (view));
 
   return scroll;
 }
@@ -667,7 +839,7 @@ GtkWidget *
 make_small_label (const char *text)
 {
   GtkWidget *label = gtk_label_new (text);
-  gtk_widget_modify_font (label, get_small_font ());
+  gtk_widget_modify_font (label, get_small_font (label));
   return label;
 }
 
@@ -880,7 +1052,61 @@ static void set_global_package_list (GList *packages,
 				     package_info_callback *selected,
 				     package_info_callback *activated);
 
-static GList *global_packages;
+static GList *global_packages = NULL;
+
+static gboolean
+global_package_list_key_pressed (GtkWidget * widget,
+				 GdkEventKey * event)
+{
+  GtkTreePath *cursor_path = NULL;
+
+  switch (event->keyval) 
+    {
+    case HILDON_HARDKEY_LEFT:
+      return TRUE;
+      break;
+    case HILDON_HARDKEY_RIGHT:
+      return TRUE;
+      break;
+    case HILDON_HARDKEY_UP:
+      // we set the focus to the last button of the main_trail
+      gtk_tree_view_get_cursor (GTK_TREE_VIEW (widget), &cursor_path, NULL);
+        
+      if (cursor_path) 
+        {
+          if (!gtk_tree_path_prev (cursor_path))
+            {
+              GList *children = NULL;
+
+              children = 
+                gtk_container_get_children (GTK_CONTAINER (get_main_trail()));
+
+              if (children) 
+                {
+                  GList *last_child = g_list_last (children);
+
+                  while (last_child && 
+                         ((!GTK_WIDGET_CAN_FOCUS (last_child->data)) ||
+                         (!GTK_WIDGET_IS_SENSITIVE (last_child->data))))
+                    last_child = g_list_previous (last_child);
+
+                  if (last_child)
+                    gtk_widget_grab_focus (GTK_WIDGET (last_child->data));
+                  
+                  g_list_free (children);
+                  gtk_tree_path_free(cursor_path);
+                  return TRUE;
+                }
+            }
+
+          gtk_tree_path_free(cursor_path);
+        }
+    
+      break;
+    }
+
+  return FALSE;
+}
 
 GtkWidget *
 make_global_package_list (GList *packages,
@@ -911,33 +1137,25 @@ make_global_package_list (GList *packages,
 
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tree), TRUE);
 
+  column = gtk_tree_view_column_new ();
+
   renderer = gtk_cell_renderer_pixbuf_new ();
   g_object_set (renderer, "yalign", 0.0, NULL);
-  gtk_tree_view_insert_column_with_data_func (GTK_TREE_VIEW (tree),
-					      -1,
-					      NULL,
-					      renderer,
-					      global_icon_func,
-					      tree,
-					      NULL);
-  column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree), 0);
-  gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
-  gtk_tree_view_column_set_expand (column, FALSE);
-  gtk_tree_view_column_set_fixed_width (column, 30);
+  gtk_cell_renderer_set_fixed_size (renderer, 30, -1);
+  gtk_tree_view_column_pack_start (column, renderer, FALSE);
+  gtk_tree_view_column_set_cell_data_func (column, renderer, global_icon_func, tree, NULL);
 
   renderer = gtk_cell_renderer_text_new ();
   g_object_set (renderer, "yalign", 0.0, NULL);
-  gtk_tree_view_insert_column_with_data_func (GTK_TREE_VIEW (tree),
-					      -1,
-					      NULL,
-					      renderer,
-					      global_name_func,
-					      tree,
-					      NULL);
-  column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree), 1);
+  gtk_tree_view_column_pack_end (column, renderer, TRUE);
+  gtk_tree_view_column_set_cell_data_func (column, renderer, global_name_func, tree, NULL);
+
+  gtk_tree_view_insert_column (GTK_TREE_VIEW (tree), column, -1);
+
+  column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree), 0);
 
   // Setting the sizing of this columne to FIXED but not specifying
-  // the widh is a workaround for some bug in GtkTreeView.  If we
+  // the width is a workaround for some bug in GtkTreeView.  If we
   // don't do this, the name will not get ellipsized and the size
   // and/or version columns might disappear completely.  With this
   // workaround, the name gets properly elipsized.
@@ -955,7 +1173,7 @@ make_global_package_list (GList *packages,
 					      global_version_func,
 					      tree,
 					      NULL);
-  column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree), 2);
+  column = gtk_tree_view_get_column (GTK_TREE_VIEW (tree), 1);
   gtk_tree_view_column_set_alignment (column, 1.0);
   gtk_tree_view_column_set_expand (column, FALSE);
 
@@ -995,6 +1213,9 @@ make_global_package_list (GList *packages,
   g_signal_connect (tree, "row-activated", 
 		    G_CALLBACK (global_row_activated), NULL);
 
+  g_signal_connect (tree, "key-press-event",
+                    G_CALLBACK (global_package_list_key_pressed), NULL);
+
   GtkWidget *menu = create_package_menu (op_label);
   gtk_widget_show_all (menu);
   gtk_widget_tap_and_hold_setup (tree, menu, NULL,
@@ -1004,6 +1225,11 @@ make_global_package_list (GList *packages,
 
   grab_focus_on_map (tree);
 
+  GtkTreePath *root = gtk_tree_path_new_root ();
+  gtk_tree_view_set_cursor (GTK_TREE_VIEW (tree), root, NULL, FALSE);
+  gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (tree), root, NULL, FALSE, 0, 0);
+  gtk_tree_path_free (root);
+
   return scroller;
 }
 
@@ -1028,15 +1254,16 @@ set_global_package_list (GList *packages,
   global_activation_callback = activated;
   global_packages = packages;
   
+  int pos = 0;
   for (GList *p = global_packages; p; p = p->next)
     {
-      GtkTreeIter iter;
       package_info *pi = (package_info *)p->data;
       pi->model = GTK_TREE_MODEL (global_list_store);
-      gtk_list_store_append (global_list_store, &pi->iter);
-      gtk_list_store_set (global_list_store, &pi->iter,
-			  0, pi,
-			  -1);
+      gtk_list_store_insert_with_values (global_list_store, &pi->iter,
+					 pos,
+					 0, pi,
+					 -1);
+      pos++;
     }
 }
 
@@ -1053,7 +1280,7 @@ global_package_info_changed (package_info *pi)
     global_row_changed (&pi->iter);
 }
 
-static GtkWidget *global_section_list;
+static GtkWidget *global_section_list = NULL;
 static section_activated *global_section_activated;
 
 static void
@@ -1064,6 +1291,22 @@ section_clicked (GtkWidget *widget, gpointer data)
     global_section_activated (si);
 }
 
+static gboolean
+scroll_to_widget (GtkWidget *w, GdkEvent *, gpointer data)
+{
+  GtkWidget *scroller = (GtkWidget *)data;
+  GtkAdjustment *adj =
+    gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scroller));
+
+  // XXX - this assumes that the adjustement unit is 'pixels'.
+
+  gtk_adjustment_clamp_page (adj,
+			     w->allocation.y,
+			     w->allocation.y + w->allocation.height);
+
+  return FALSE;
+}
+
 GtkWidget *
 make_global_section_list (GList *sections, section_activated *act)
 {
@@ -1084,6 +1327,8 @@ make_global_section_list (GList *sections, section_activated *act)
 
   bool first_button = true;
 
+  scroller = gtk_scrolled_window_new (NULL, NULL);
+
   for (GList *s = sections; s; s = s ->next)
     {
       section_info *si = (section_info *)s->data;
@@ -1106,9 +1351,11 @@ make_global_section_list (GList *sections, section_activated *act)
 
       osso_gtk_button_set_detail_from_attach_flags
 	(GTK_BUTTON (btn), OssoGtkButtonAttachFlags (f));
+
+      g_signal_connect (btn, "focus-in-event",
+			G_CALLBACK (scroll_to_widget), scroller);
     }
 
-  scroller = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroller),
 				  GTK_POLICY_NEVER,
 				  GTK_POLICY_AUTOMATIC);
@@ -1222,6 +1469,7 @@ fcd_response (GtkDialog *dialog, gint response, gpointer clos)
 
   char *uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
 
+  pop_dialog_parent ();
   gtk_widget_destroy (GTK_WIDGET (dialog));
 
   if (response == GTK_RESPONSE_OK)
@@ -1242,13 +1490,14 @@ show_deb_file_chooser (void (*cont) (char *uri, void *data),
   GtkFileFilter *filter;
 
   fcd = hildon_file_chooser_dialog_new_with_properties
-    (get_main_window (),
+    (get_dialog_parent (),
      "action",            GTK_FILE_CHOOSER_ACTION_OPEN,
      "title",             _("ai_ti_select_package"),
      "empty_text",        _("ai_ia_select_package_no_packages"),
      "open_button_text",  _("ai_bd_select_package"),
      NULL);
   gtk_window_set_modal (GTK_WINDOW (fcd), TRUE);
+  push_dialog_parent (fcd);
 
   filter = gtk_file_filter_new ();
   gtk_file_filter_add_mime_type (filter, "application/x-deb");
@@ -1265,6 +1514,7 @@ show_deb_file_chooser (void (*cont) (char *uri, void *data),
 
 void
 show_file_chooser_for_save (const char *title,
+			    GtkWindow *parent,
 			    const char *default_filename,
 			    void (*cont) (char *uri, void *data),
 			    void *data)
@@ -1276,10 +1526,11 @@ show_file_chooser_for_save (const char *title,
   GtkWidget *fcd;
 
   fcd = hildon_file_chooser_dialog_new_with_properties
-    (get_main_window (),
+    (get_dialog_parent (),
      "action",            GTK_FILE_CHOOSER_ACTION_SAVE,
      "title",             title,
      NULL);
+  push_dialog_parent (fcd);
   gtk_window_set_modal (GTK_WINDOW (fcd), TRUE);
 
   gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (fcd), default_filename);
@@ -1367,6 +1618,7 @@ b64decode (const unsigned char *str, GdkPixbufLoader *loader)
 		  g_error_free (error);
 		  return;
 		}
+
 	      loader_ptr = loader_buf;
 	    }
         }
@@ -1421,18 +1673,24 @@ static void *copy_cont_data;
 static char *copy_local;
 static char *copy_target;
 static char *copy_tempdir;
+static GnomeVFSHandle *copy_vfs_handle;
 
 static void
-call_copy_cont (bool success, bool show_error = true)
+call_copy_cont (GnomeVFSResult result)
 {
   hide_progress ();
 
-  if (!success && show_error)
+  if (result == GNOME_VFS_ERROR_IO)
+    annoy_user (dgettext ("hildon-common-strings",
+			  "sfil_ni_cannot_open_no_connection"));
+  else if (result == GNOME_VFS_ERROR_CANCELLED)
+    ;
+  else if (result != GNOME_VFS_OK)
     annoy_user (_("ai_ni_operation_failed"));
-    
+
   if (copy_cont)
     {
-      if (success)
+      if (result == GNOME_VFS_OK)
 	copy_cont (copy_target, copy_cont_data);
       else
 	{
@@ -1461,7 +1719,22 @@ copy_progress (GnomeVFSAsyncHandle *handle,
 
   if (info->phase == GNOME_VFS_XFER_PHASE_COMPLETED)
     {
-      call_copy_cont (!progress_was_cancelled (), false);
+      struct stat buf;
+
+      if (stat (copy_target, &buf) < 0)
+	{
+	  /* If a obex connection is refused before the downloading is
+	     started, ovu_async_xfer seems to report success without
+	     actually creating the file.  We treat that situation as
+	     an I/O error.
+	  */
+	  call_copy_cont (GNOME_VFS_ERROR_IO);
+	}
+      else
+	call_copy_cont (progress_was_cancelled ()
+			? GNOME_VFS_ERROR_CANCELLED
+			: GNOME_VFS_OK);
+
       gnome_vfs_async_cancel (handle);
     }
 
@@ -1473,7 +1746,7 @@ copy_progress (GnomeVFSAsyncHandle *handle,
     }
   else if (info->status == GNOME_VFS_XFER_PROGRESS_STATUS_VFSERROR)
     {
-      call_copy_cont (false);
+      call_copy_cont (info->vfs_status);
       gnome_vfs_async_cancel (handle);
       return GNOME_VFS_XFER_ERROR_ACTION_ABORT;
     }
@@ -1496,7 +1769,7 @@ do_copy (const char *source, GnomeVFSURI *source_uri,
   target_uri = gnome_vfs_uri_new (target);
   if (target_uri == NULL)
     {
-      call_copy_cont (false);
+      call_copy_cont (GNOME_VFS_ERROR_NO_MEMORY);
       return;
     }
 
@@ -1519,23 +1792,32 @@ do_copy (const char *source, GnomeVFSURI *source_uri,
 			   NULL);
 
   if (result != GNOME_VFS_OK)
-    call_copy_cont (false);
+    call_copy_cont (result);
 }
 
 void
-localize_file (const char *uri,
-	       void (*cont) (char *local, void *data),
-	       void *data)
+localize_file_and_keep_it_open (const char *uri,
+				void (*cont) (char *local, void *data),
+				void *data)
 {
+  if (copy_cont != NULL)
+    {
+      add_log ("Unexpected reentry\n");
+      if (cont)
+	cont (NULL, data);
+      return;
+    }
+
   copy_cont = cont;
   copy_cont_data = data;
   copy_target = NULL;
   copy_local = NULL;
   copy_tempdir = NULL;
+  copy_vfs_handle = NULL;
 
   if (!gnome_vfs_init ())
     {
-      call_copy_cont (false);
+      call_copy_cont (GNOME_VFS_ERROR_GENERIC);
       return;
     }
 
@@ -1543,7 +1825,7 @@ localize_file (const char *uri,
 
   if (vfs_uri == NULL)
     {
-      call_copy_cont (false);
+      call_copy_cont (GNOME_VFS_ERROR_NO_MEMORY);
       return;
     }
 
@@ -1556,9 +1838,20 @@ localize_file (const char *uri,
   const gchar *scheme = gnome_vfs_uri_get_scheme (vfs_uri);
   if (scheme && !strcmp (scheme, "file"))
     {
-      const gchar *path = gnome_vfs_uri_get_path (vfs_uri);
-      copy_target = gnome_vfs_unescape_string (path, NULL);
-      call_copy_cont (true);
+      /* Open the file to protect against unmounting of the MMC, etc.
+       */
+      GnomeVFSResult result;
+      
+      result = gnome_vfs_open_uri (&copy_vfs_handle, vfs_uri,
+				   GNOME_VFS_OPEN_READ);
+      if (result != GNOME_VFS_OK)
+	call_copy_cont (result);
+      else
+	{
+	  const gchar *path = gnome_vfs_uri_get_path (vfs_uri);
+	  copy_target = gnome_vfs_unescape_string (path, NULL);
+	  call_copy_cont (GNOME_VFS_OK);
+	}
     }
   else
     {
@@ -1578,12 +1871,12 @@ localize_file (const char *uri,
       if (copy_tempdir == NULL)
 	{
 	  add_log ("Can not create %s: %m", copy_tempdir);
-	  call_copy_cont (false);
+	  call_copy_cont (GNOME_VFS_ERROR_GENERIC);
 	}
       else if (chmod (copy_tempdir, 0755) < 0)
 	{
 	  add_log ("Can not chmod %s: %m", copy_tempdir);
-	  call_copy_cont (false);
+	  call_copy_cont (GNOME_VFS_ERROR_GENERIC);
 	}
       else
 	{
@@ -1595,6 +1888,8 @@ localize_file (const char *uri,
 	  do_copy (uri, vfs_uri, copy_target);
 	}
     }
+      
+  gnome_vfs_uri_unref (vfs_uri);
 }
 
 void
@@ -1605,6 +1900,12 @@ cleanup_temp_file ()
      has seen the report already.
   */
 
+  if (copy_vfs_handle)
+    {
+      gnome_vfs_close (copy_vfs_handle);
+      copy_vfs_handle = NULL;
+    }
+
   if (copy_local)
     {
       if (unlink (copy_local) < 0)
@@ -1614,6 +1915,8 @@ cleanup_temp_file ()
 
       g_free (copy_local);
       g_free (copy_tempdir);
+      copy_local = NULL;
+      copy_tempdir = NULL;
     }
 }
 
@@ -1671,15 +1974,22 @@ run_cmd (char **argv,
   g_child_watch_add (child_pid, reap_process, c);
 }
 
-int
-all_white_space (const char *text)
+const char *
+skip_whitespace (const char *str)
 {
-  while (*text)
-    if (!isspace (*text++))
-      return 0;
-  return 1;
+  while (isspace (*str))
+    str++;
+  return str;
 }
 
+bool
+all_white_space (const char *str)
+{
+  return (*skip_whitespace (str)) == '\0';
+}
+
+static ConIcConnection *connection_object = NULL;
+
 struct en_closure {
   void (*callback) (bool success, void *data);
   void *data;
@@ -1711,30 +2021,27 @@ ensure_network_cont (bool success)
 }
 
 static void
-iap_callback (struct iap_event_t *event, void *arg)
+iap_callback (ConIcConnection *connection,
+	      ConIcConnectionEvent *event,
+	      gpointer user_data)
 {
   bool success = false;
-
-  switch (event->type)
+  
+  switch (con_ic_connection_event_get_status (event))
     {
-    case OSSO_IAP_CONNECTED:
-      // add_log ("OSSO_IAP_CONNECTED: %s\n", event->iap_name);
+    case CON_IC_STATUS_CONNECTED:
+      // add_log ("CON_IC_STATUS_CONNECTED\n");
       success = true;
       break;
 
-    case OSSO_IAP_DISCONNECTED:
-      // add_log ("OSSO_IAP_DISCONNECTED: %s\n", event->iap_name);
+    case CON_IC_STATUS_DISCONNECTED:
+      // add_log ("CON_IC_STATUS_DISCONNECTED\n");
       if (current_status_operation == op_downloading)
 	cancel_apt_worker ();
       break;
 
-    case OSSO_IAP_ERROR:
-      add_log ("IAP Error: %x %s.\n", -event->u.error_code, event->iap_name);
-      annoy_user (_("ai_ni_operation_failed"));
-      break;
-
     default:
-      add_log ("IAP Error: unexpected event type %d\n", event->type);
+      add_log ("ConIc Error: unexpected event type\n");
       annoy_user (_("ai_ni_operation_failed"));
       break;
     }
@@ -1757,17 +2064,25 @@ ensure_network (void (*callback) (bool success, void *data), void *data)
       ensure_network_cont (true);
       return;
     }
-  
-  if (osso_iap_cb (iap_callback) == OSSO_OK)
+
+  if (connection_object == NULL)
     {
-      if (osso_iap_connect (OSSO_IAP_ANY, OSSO_IAP_REQUESTED_CONNECT, NULL)
-	  == OSSO_OK)
-	{
-	  show_progress (dgettext ("osso-browser-ui",
-				   "weba_pb_clearing_connecting"));
-	  return;
-	}
+      connection_object = con_ic_connection_new ();
+      g_signal_connect (connection_object, "connection-event",
+			G_CALLBACK (iap_callback), NULL);
     }
+  
+  if (con_ic_connection_connect (connection_object, CON_IC_CONNECT_FLAG_NONE))
+    {
+      if (ui_version < 2)
+	show_progress (dgettext ("osso-browser-ui",
+				 "weba_pb_clearing_connecting"));
+      else
+	show_progress (_("ai_nw_connecting"));
+      return;
+    }
+  else
+    add_log ("con_ic_connection_connect failed\n");
 
   annoy_user (_("ai_ni_operation_failed"));
   ensure_network_cont (false);
@@ -1794,7 +2109,7 @@ get_http_proxy ()
     {
       char *user = NULL;
       char *password = NULL;
-      char *host;
+      char *host = NULL;
       gint port;
 
       if (gconf_client_get_bool (conf, "/system/http_proxy/use_authentication",
@@ -1821,6 +2136,10 @@ get_http_proxy ()
 	}
       else
 	proxy = g_strdup_printf ("http://%s:%d", host, port);
+
+      g_free (user);
+      g_free (password);
+      g_free (host);
     }
   else
     proxy = g_strdup (getenv ("http_proxy"));
@@ -1851,12 +2170,13 @@ get_https_proxy ()
   */
   gconf_client_clear_cache (conf);
 
-  const char *host =
+  char *host =
     gconf_client_get_string (conf, "/system/proxy/secure_host", NULL);
   int port = gconf_client_get_int (conf, "/system/proxy/secure_port", NULL);
 
   if (host && host[0])
     proxy = g_strdup_printf ("http://%s:%d", host, port);
+  g_free(host);
 
   g_object_unref (conf);
 
@@ -1889,39 +2209,134 @@ gettext_alt (const char *id, const char *english)
     return tr;
 }
 
-/* XXX - There must be a better way.  Also, we don't track changes to
-         the device name.
- */
-
-static const char *btname_result = NULL;
+static char *btname = NULL;
 
 const char *
 device_name ()
 {
-  if (btname_result)
-    return btname_result;
+  if (btname != NULL)
+    return btname;
+  else
+    return dgettext ("hildon-fm", "sfil_li_folder_root");
+}
 
-  GError *error = NULL;
-  char *output = NULL;
-  btname_result = "Nokia 770";
+static void
+set_bt_name_from_message (DBusMessage *message)
+{
+  DBusMessageIter iter;
+  const char *name = NULL;
+  GtkWidget *label = NULL;
 
-  char *params[] = { "/usr/bin/btname", "-g", NULL };
+  g_return_if_fail (message != NULL);
 
-  if (!g_spawn_sync (NULL, params, NULL, GSpawnFlags(0), NULL, NULL, 
-		     &output, NULL, NULL, &error))
+  if (!dbus_message_iter_init (message, &iter))
     {
-      add_log ("can't run btname: %s", error->message);
-      g_error_free (error);
+      add_log ("message did not have argument\n");
+      return;
     }
-  else if (output && output[0] != '\0')
+  dbus_message_iter_get_basic (&iter, &name);
+
+  if (btname) 
+    g_free (btname);
+
+  btname = g_strdup (name);
+
+  label = get_device_label ();
+
+  if (label)
+    gtk_label_set_text (GTK_LABEL (label), btname);
+}
+
+static void 
+btname_received(DBusPendingCall *call, void *user_data)
+{
+  DBusMessage *message;
+  DBusError error;
+
+  g_assert (dbus_pending_call_get_completed (call));
+  message = dbus_pending_call_steal_reply (call);
+  if (message == NULL)
     {
-      /* Strip the newline.
-       */
-      output[strlen(output)-1] = '\0';
-      btname_result = output;
+      add_log ("no reply\n");
+      return;
     }
 
-  return btname_result;
+  dbus_error_init (&error);
+
+  if (dbus_set_error_from_message (&error, message))
+    {
+      add_log ("get btname: %s\n", error.message);
+      dbus_error_free (&error);
+    }
+  else   
+    set_bt_name_from_message (message);
+
+  dbus_message_unref (message);
+}
+
+static DBusHandlerResult
+handle_dbus_signal (DBusConnection *conn,
+		    DBusMessage *msg,
+		    gpointer data)
+{
+  if (dbus_message_is_signal(msg, BTNAME_SIGNAL_IF, BTNAME_SIG_CHANGED))
+    set_bt_name_from_message(msg);
+
+  return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
+
+void 
+setup_dbus ()
+{
+  static DBusConnection *conn = NULL;
+  DBusMessage *request;
+  DBusError error;
+  DBusPendingCall *call = NULL;
+
+  dbus_error_init (&error);
+  if (conn == NULL)
+    {
+      conn = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
+      if (conn == NULL)
+        {
+          add_log ("%s: %s\n", error.name, error.message);
+	  dbus_error_free (&error);
+          return;
+        }
+    }
+
+  /* Let's query initial state.  These calls are async, so they do not
+     consume too much startup time.
+   */
+  request = dbus_message_new_method_call (BTNAME_SERVICE,
+					  BTNAME_REQUEST_PATH,
+					  BTNAME_REQUEST_IF,
+					  BTNAME_REQ_GET);
+  if (request == NULL)
+    {
+      add_log ("dbus_message_new_method_call failed\n");
+      return;
+    }
+  dbus_message_set_auto_start (request, TRUE);
+
+  if (dbus_connection_send_with_reply (conn, request, &call, -1))
+    {
+      dbus_pending_call_set_notify (call, btname_received, NULL, NULL);
+      dbus_pending_call_unref (call);
+    }
+
+  dbus_message_unref (request);
+
+  dbus_connection_setup_with_g_main (conn, NULL);
+  dbus_bus_add_match (conn, BTNAME_MATCH_RULE, &error);
+  if (dbus_error_is_set(&error))
+    {
+      add_log ("dbus_bus_add_match failed: %s\n", error.message);
+      dbus_error_free (&error);
+    }
+
+  if (!dbus_connection_add_filter(conn, handle_dbus_signal, NULL, NULL))
+    add_log ("dbus_connection_add_filter failed\n");
 }
 
 static gboolean
